#ifndef __Value_h__
#define __Value_h__

#include <cadadata/CADAData.h>
#include <cadadata/Exception.h>

#include <librfc/XMLNode.hpp>
#include <librfcdal/DalDatabase.h>
#include <librfcdal/SqlColumn.h>

#include <string>
#include <vector>

namespace CADAData
{
	const int nInvalidNode = 1;
	const int nInvalidRow = 2;

	class Base;

	class CADAData_Export Value
	{
		private:
			Base* _pbase;
			std::string _strName;

		public:
			Value( Base* pbase, const std::string& strName );
			Value( const Value& value );
			virtual ~Value();

			const Value& operator=( const Value& value );

			const std::string& name() const;

			virtual Value* copy() const = 0;
			virtual void toXML( Rfc::XMLNode& nodeParent ) const = 0;
			virtual void fromXML( const Rfc::XMLNode& nodeParent ) throw( Exception ) = 0;
			virtual void fromRow( const RfcDal::DalDatabase::Row& row, const RfcDal::SqlColumn& column ) throw( Exception ) = 0;
	};

	class CADAData_Export StringValue : public Value
	{
		private:
			std::string _strValue;

		public:
			StringValue( Base* pbase, const std::string& strName, const std::string& strValue = std::string() );
			StringValue( const StringValue& value );
			virtual ~StringValue();

			const StringValue& operator=( const StringValue& value );
			const StringValue& operator=( const std::string& strValue );

			const std::string& value() const;

			virtual Value* copy() const;
			virtual void toXML( Rfc::XMLNode& nodeParent ) const;
			virtual void fromXML( const Rfc::XMLNode& nodeParent ) throw( Exception );
			virtual void fromRow( const RfcDal::DalDatabase::Row& row, const RfcDal::SqlColumn& column ) throw( Exception );
	};

	class CADAData_Export IntValue : public Value
	{
		private:
			int _nValue;

		public:
			IntValue( Base* pbase, const std::string& strName, int nValue = 0 );
			IntValue( const IntValue& value );
			virtual ~IntValue();

			const IntValue& operator=( const IntValue& value );
			const IntValue& operator=( int nValue );

			int value() const;

			virtual Value* copy() const;
			virtual void toXML( Rfc::XMLNode& nodeParent ) const;
			virtual void fromXML( const Rfc::XMLNode& nodeParent ) throw( Exception );
			virtual void fromRow( const RfcDal::DalDatabase::Row& row, const RfcDal::SqlColumn& column ) throw( Exception );
	};

	class CADAData_Export DoubleValue : public Value
	{
		private:
			double _dValue;

		public:
			DoubleValue( Base* pbase, const std::string& strName, double dValue = 0. );
			DoubleValue( const DoubleValue& value );
			virtual ~DoubleValue();

			const DoubleValue& operator=( const DoubleValue& value );
			const DoubleValue& operator=( double dValue );

			double value() const;

			virtual Value* copy() const;
			virtual void toXML( Rfc::XMLNode& nodeParent ) const;
			virtual void fromXML( const Rfc::XMLNode& nodeParent ) throw( Exception );
			virtual void fromRow( const RfcDal::DalDatabase::Row& row, const RfcDal::SqlColumn& column ) throw( Exception );
	};

	class CADAData_Export ValuePtr
	{
		public:
			typedef enum { typeString, typeInt, typeDouble } ValueType;

		private:
			Value* _pvalue;
			ValueType _type;

		public:
			ValuePtr( StringValue& value );
			ValuePtr( IntValue& value );
			ValuePtr( DoubleValue& value );
			ValuePtr( ValuePtr& value );
			virtual ~ValuePtr();

			const ValuePtr& operator=( const ValuePtr& value );

			ValueType type() const;
			const Value& value() const;
			Value& value();
	};

	typedef std::vector<ValuePtr> ValuePtrList;
};

#endif /* !__Value_h__ */
